-- main.lua
-- ****** Comment *****************************************************
-- Revision: 2.1
-- Issued  : Nov.11, 2024
-- Auther  : Tanzai
-- Contents: This script controlls a plugin, "H3-22S_launch" on X-Plane 12.
--           This Plugin shall be stored in /plugins/ folder under airplane
--           folder.
-- Notice 1: The package may be redistributed for non commercial purposes
--           only; derivatives MAY NOT be redistributed.
-- Notice 2: Do not resize X-Plane window during flight to prevent incorrrect
--           display. At any rate you can restore the window by pressing a menu 
--           "Reload theCurrent Airplane (Skip Art Reload)" under "Developer".
-- *******************************************************************
-- Custom slider setting
--   showPart[10] = Boosters smoke/jet display    0:hide  1:show
--   showPart[11] = Stage 1 jet display           0:hide  1:show
--   showPart[12] = Stage 2 jet display           0:hide  1:show

setAircraftPanelRendering(false)
set3DRendering(false)
setInteractivity(false)

-- Subwindow setting  ------------------------------------------
winW = globalPropertyi("sim/graphics/view/window_width")
winH = globalPropertyi("sim/graphics/view/window_height")
local starter = contextWindow {
    position		= { 0, get(winH)-80, 1200, 80},
    noResize		= false,
    visible			= true,
	proportional	= false,
    noBackground    = true,
	layer			= SASL_CW_LAYER_FLOATING_WINDOWS,
    noDecore        = true,
    noMove          = true,
    components		= {simBoard  {position={0, 0,1200, 75},},
 					   simLiftoff{position={0, 0,1200, 34},},
					  },}

local popup = contextWindow {
	position 		= {(get(winW)/2)-500, (get(winH)/2)+180, 1000, 100},
	noResize 		= true,
	visible 		= true,
	noBackground 	= true,
	layer			= SASL_CW_LAYER_FLOATING_WINDOWS,
	noDecore 		= true,
	noMove 			= true,
	components 		= {simPopup{position={0, 0, 1000, 100},},},
					  }

-- Variables setting  ---------------------------------------------------
defineProperty("startIsOn", 0)
defineProperty("simStage", 0)
defineProperty("resetIsOn", 0)
defineProperty("t3", 0)
defineProperty("targetH", 400)
defineProperty("headPsi", 90)                                  -- deg 方向
defineProperty("pitchThe", 0)                                  -- deg 迎角
defineProperty("rollPhi", 0)                                   -- deg 左右傾
defineProperty("Note1", " ")                                   -- 説明書き、１行目
defineProperty("crrVelo", 0.0)                                 -- 1秒毎の速度
defineProperty("endT3", 0)
defineProperty("endVelo", 0.0)
defineProperty("endPosH", 0.0)
defineProperty("endLon", 0.0)
defineProperty("endLat", 0.0)

addSearchResourcesPath ( moduleDirectory .. "/configuration/" )
hideBooster1= createGlobalPropertyi("H3/hide/Booster1",   0)  -- 0:show 1:hide
hideBooster2= createGlobalPropertyi("H3/hide/Booster2",   0)  -- 0:show 1:hide
hideFirstS  = createGlobalPropertyi("H3/hide/FirstStage", 0)  -- 0:show 1:hide
hideSecondC = createGlobalPropertyi("H3/hide/SecondCov",  0)  -- 0:show 1:hide
hideFairingL= createGlobalPropertyi("H3/hide/FairingSL",  0)  -- 0:show 1:hide
hideFairingR= createGlobalPropertyi("H3/hide/FairingSR",  0)  -- 0:show 1:hide
hideSecondS = createGlobalPropertyi("H3/hide/SecondStage",0)  -- 0:show 1:hide
LauncherObj = sasl.loadObject( "H3_Launch_pad.obj" )
H3_coundown = sasl.al.loadSample ( "H3_countdown.wav" )
H3_sound = sasl.al.loadSample ( "H3_sound.wav" )
as = globalPropertyd("sim/flightmodel/position/true_airspeed")   -- n, m/s 速度
elv = globalPropertyd("sim/flightmodel/position/elevation")      -- n, deg 高度
lat = globalPropertyf("sim/flightmodel/position/latitude")       -- n, deg 緯度
lon = globalPropertyf("sim/flightmodel/position/longitude")      -- n, deg 経度
local quat = globalPropertyfa("sim/flightmodel/position/q")      -- y, 四元数
local vx = globalPropertyf("sim/flightmodel/position/local_vx")  -- y, m/s X軸
local vy = globalPropertyf("sim/flightmodel/position/local_vy")  -- y, m/s Y軸
local vz = globalPropertyf("sim/flightmodel/position/local_vz")  -- y, m/s Z軸
-- local planeX = globalPropertyf("sim/flightmodel/position/local_x")
-- local planeY = globalPropertyf("sim/flightmodel/position/local_y")
-- local planeZ = globalPropertyf("sim/flightmodel/position/local_z")
local vRoll = globalPropertyf("sim/flightmodel/position/P")      -- deg/sec
local vPitch= globalPropertyf("sim/flightmodel/position/Q")      -- deg/sec
local vYaw  = globalPropertyf("sim/flightmodel/position/R")      -- deg/sec
sim_time = globalPropertyf("sim/operation/misc/frame_rate_period")
local Booster_release = createGlobalPropertyf("H3/Booster/release", 0)
local First_release   = createGlobalPropertyf("H3/First/release", 0)
local Second_release  = createGlobalPropertyf("H3/Second/release", 0)
local showPart = globalPropertyia("sim/cockpit2/switches/custom_slider_on")
showMenu = sasl.findCommand("sim/operation/show_menu")
simTime = sasl.createTimer()
oldVelo = {0.0,0.0,0.0}
xFactor = {}
yFactor = {}
vFactor = {}
fuelMass = {}
iniMass = {}
totalMass = {}

-- Function to set rocket parameters  -----------------------------
function setRocket()
	if get(targetH) == 100 then                    -- 目標高度(km)
		-- Spec   Booster   1st    2nd  Orbiter
		tStart  ={   0  ,  110  ,  138 ,  0   }  -- 燃焼開始時間(整数) sec
		firingT ={ 105  ,   15  ,  370 ,  0   }  -- 燃焼時間(整数) sec
		angThe  ={ -10.0,  -80.0, -87.0, -89.0}  -- 迎角 deg
		targetV = 7844                           -- 目標速度(m/sec)
	elseif get(targetH) == 200 then              -- 目標高度(km)
		-- Spec    Booster  1st      2nd  Orbiter
		tStart  ={   0  ,  110  ,  137 ,  0   }  -- 燃焼開始秒 sec
		firingT ={ 105  ,   15  ,  370 ,  0   }  -- 燃焼時間 sec
		angThe  ={ -10.0,  -20.0, -50.0, -89.0}  -- 迎角 deg
		targetV = 7784                           -- 目標速度(m/sec)
	elseif get(targetH) == 400 then              -- 目標高度(km)
		-- Spec    Booster   1st     2nd  Orbiter
		tStart  ={   0  ,  110  , 206  ,  0   }  -- 燃焼開始時間(整数) sec
		firingT ={ 105  ,   84  , 370  ,  0   }  -- 燃焼時間(整数) sec
		angThe  ={ -10.0,  -40.0, -60.0, -89.0}  -- 迎角 deg
		targetV = 7668                           -- 目標速度(m/sec)
	elseif get(targetH) == 600 then              -- 目標高度(km)
		-- Spec    Booster   31st     2nd  Orbiter
		tStart  ={   0  ,  110  , 246  ,  0   }  -- 燃焼開始時間(整数) sec
		firingT ={ 105  ,   80  , 370  ,  0   }  -- 燃焼時間(整数) sec
		angThe  ={ -10.0,  -25.0, -60.0, -89.0}  -- 迎角 deg
		targetV = 7558                             -- 目標速度(m/sec)
	elseif get(targetH) == 800 then                -- 目標高度(km)
		-- Spec    Booster   1st     2nd  Orbiter
		tStart  ={   0  ,  110  , 256  ,  0   }  -- 燃焼開始時間(整数) sec
		firingT ={ 105  ,   95  , 370  ,  0   }  -- 燃焼時間(整数) sec
		angThe  ={ -10.0,  -20.0, -30.0, -89.0}  -- 迎角 deg
		targetV = 7452                           -- 目標速度(m/sec)
	end
	-- 最初の6秒間は迎角90°となります。
	-- Boost終了からFirst点火まで4秒以上開けること。
	-- Firstの燃焼時間は12秒以上の事。
	-- First終了からSecond点火まで１２秒以上開けること。
	-- Second燃焼中に高度がtargetHを超えたらに迎角を0°に変更する。
	-- Second燃焼中に速度がtargetVを超えたら停止する。
	-- 燃焼時間(firingT)の上限は、firingT={ 105, 153, 370, 0 }

	-- Spec    Booster   1st     2nd  Orbiter
	gLsp    ={ 2279.3, 4165.0, 4390.4,  0.0 }  -- 有効排気速度(=g*Lsp) m/s
	dryMass ={   37.4,   12.5,    3.0,  3.0 }  -- 空筐体重 ton
	fuelRate={  1.082, 0.7092, 0.03108, 0.0 }  -- 燃料消費速度 ton/sec

	for m = 1,4 do
		fuelMass[m] = fuelRate[m] * firingT[m]
		iniMass[m] = dryMass[m] + fuelMass[m]
	end
	totalMass[1] = iniMass[1] + iniMass[2] + iniMass[3] + iniMass[4]
	totalMass[2] = iniMass[2] + iniMass[3] + iniMass[4]
	totalMass[3] = iniMass[3] + iniMass[4]
end

-- -------  Other functions  ----------------------------------------------
function setOrient()
	psRad = math.pi / 360 * get(headPsi)                          -- rad 方向
	thRad = math.pi / 360 * get(pitchThe)                         -- rad 迎角
	phRad = math.pi / 360 * get(rollPhi)                          -- rad 左右傾
	q0 = math.cos(psRad)*math.cos(thRad)*math.cos(phRad) + math.sin(psRad)*math.sin(thRad)*math.sin(phRad)
	q1 = math.cos(psRad)*math.cos(thRad)*math.sin(phRad) - math.sin(psRad)*math.sin(thRad)*math.cos(phRad)
	q2 = math.cos(psRad)*math.sin(thRad)*math.cos(phRad) + math.sin(psRad)*math.cos(thRad)*math.sin(phRad)
	q3 = -math.cos(psRad)*math.sin(thRad)*math.sin(phRad) + math.sin(psRad)*math.cos(thRad)*math.cos(phRad)
	set(quat, {q0,q1,q2,q3})
end

function setSpeed(n)                 -- Setting speed by Tsiolkovsky's equation
	xFactor[n] =-math.sin(math.rad(get(pitchThe)) * math.sin(math.rad(get(headPsi))))
	yFactor[n] = math.sin(math.rad(get(pitchThe)) * math.cos(math.rad(get(headPsi))))
	vFactor[n] = math.cos(math.rad(get(pitchThe)))
	-------------
	massRatio= 1.0 - fuelMass[n]*(get(t3)-tStart[n])/firingT[n]/totalMass[n]
	deltaVelo= gLsp[n] * math.log( 1 / massRatio )
	vInX = (oldVelo[n] + deltaVelo) * xFactor[n]         -- 南北方向の係数
	vInY = (oldVelo[n] + deltaVelo) * yFactor[n]         -- 東西方向の係数
	vInH = (oldVelo[n] + deltaVelo) * vFactor[n] - 9.8   -- 垂直方向係数、重力を加味する
	set(vx, vInX)
	set(vz, vInY)
	set(vy, vInH)
	vDelta = (vInX^2 + vInY^2 + vInH^2)^0.5
	set(crrVelo, vDelta)                                 -- crrVelo: 1秒毎の速度
end

function setFreeFall(n)                              -- Setting free fall speed
	vInH = vInH - 9.8                                -- 重力を加味
	vDelta = (vInX^2 + vInY^2 + vInH^2)^0.5
	set(crrVelo, ( oldVelo[n] + vDelta ))            -- crrVelo: 1秒毎の速度
end

function stayRocketThere()
	set(vx, 0.0)
	set(vy, 0.0)
	set(vz, 0.0)
end

function stopRotation()
	set(vRoll, 0.0)
	set(vPitch, 0.0)
	set(vYaw, 0.0)
end

function iniValSet()                     -- Function to set initial value
	set(t3, -6)
	t4 = -5
	set(headPsi, 90) 
	set(pitchThe, 0)                            -- 迎角90°(z軸が水平)の状態
	set(rollPhi, 0)
	set(Note1, "")
	set(showPart, 0, 10)                        -- No show Boosters smoke/jet
	set(showPart, 0, 11)                        -- No show Stage 1 jet
	set(showPart, 0, 12)                        -- No show Stage 2 jet
	set(vx, 0.0)
	set(vz, 0.0)
	vInX = 0.0
	vInY = 0.0
	vInH = 0.0
	Action1 = 0
	Action2 = 0
	Action3 = 0
	Action4 = 0
	Action5 = 0
	Action6 = 0
	Action7 = 0
	set(Booster_release, 0)
	set(First_release, 0)
	set(Second_release, 0)
	set(hideBooster1,0)
	set(hideBooster2,0)
	set(hideFirstS,  0)
	set(hideSecondC, 0)
	set(hideFairingL,0)
	set(hideFairingR,0)
	set(hideSecondS, 0)
	state = 0
end

function recordParam()
	set(endT3, get(t3))
	set(endVelo, math.floor(get(crrVelo)))
	set(endPosH, math.floor(get(elv)/100)/10)
	set(endLon, get(lon))
	set(endLat, get(lat))
end

-- Main control ----------------------------------------------------
set(simStage, 0)
set(startIsOn, 0)
set(resetIsOn, 0)
iniValSet()                                     -- Initialize values
setRocket()                                     -- Set rocket parameters

function update()
	if get(simStage) == 0 then
		stayRocketThere()

		if get(startIsOn) == 1 then            -- "Liftoff" pressed 
			iniLon = get(lon)
			iniLat = get(lat)
			iniElv = get(elv)
			iniPsi = get(headPsi)
			sasl.resetTimer( simTime )
			sasl.startTimer( simTime )
			set(simStage, 1)
			nnn = 0
			set(startIsOn, 0)
			iniValSet()                       -- Initialize values
			setRocket()                       -- Set rocket parameters
			
		elseif get(resetIsOn) == 1 then                -- "Reset" pressed 
			sasl.commandOnce( viewSet )
			sasl.stopTimer( simTime )
			set(startIsOn, 0)
			set(resetIsOn, 0)
			set(headPsi, 90)
			sasl.placeUserAtLocation(iniLat,iniLon,iniElv,iniPsi,0.0)
			iniValSet()                                -- Initialize values
			setRocket()                                -- Set rocket parameters
		end

	elseif get(simStage) == 1 then                     -- Simulation starts
		t0 = sasl.getElapsedSeconds( simTime )
		set(t3, math.floor(t0 - 5))
		
		if get(t3) == t4 then                          -- 秒毎に以下を実施
			t4 = t4 + 1
			if nnn == 0 then
				sasl.al.playSample( H3_coundown )
				nnn = 1
			end
			if get(t3) < tStart[1] then
				stayRocketThere()
				oldVelo[1] = 0.0
				set(Note1, "Countdown starts")
			elseif get(t3) == tStart[1] then           -- Booster fires
				tEnd1 = tStart[1] + firingT[1]
				set(vx, 0.0)
				set(vz, 0.0)
				sasl.al.playSample( H3_sound )
				set(showPart, 1, 10)                   -- Boosters smoke/jet on
				set(Note1, "Lifting up by Boosters untill T= 105 sec")
			elseif get(t3) < tStart[1] + 6 then        -- Booster firing
				setSpeed(1)
				set(vx, 0.0)
				set(vz, 0.0)	
			elseif get(t3) == tStart[1] + 6 then
				setSpeed(1)
				Action1 = 1                        -- Tilt rocket a little	
			elseif get(t3) < tEnd1 then                -- Booster firing
				setSpeed(1)
			elseif get(t3) == tEnd1 then
				setSpeed(1)
				set(showPart, 0, 10)               -- Boosters smoke/jet off
				Action2 = 1                        -- Booster animation start
			elseif get(t3) < tStart[2] then        -- Booster stops
				setFreeFall(1)
				set(Note1, "Boosters released")
 			elseif get(t3) == tStart[2] then             -- 1st Stage fires
				tEnd2 = tStart[2] + firingT[2]
				oldVelo[2] = get(crrVelo)
				set(showPart, 1, 11)                     -- Stage 1 jet on
				Action3 = 1                         -- Change pitchThe in 5 sec
				set(Note1, "1st Stage engine running")
			elseif get(t3) < tEnd2 then                  -- 1st Stage firing
				setSpeed(2)
			elseif get(t3) == tEnd2 then                 -- 1st Stage ends
				setSpeed(2)
				set(showPart, 0, 11)                     -- Stage 1 jet off
				Action4 = 1                         -- animation Stage 1 release
			elseif get(t3) < tStart[3] then              -- 1st Stage stops
				setFreeFall(2)
				set(Note1, "1st Stage engine cutoff")
			elseif get(t3) == tStart[3] then             -- 2nd Stage fires
				tEnd3 = tStart[3] + firingT[3]
				oldVelo[3] = get(crrVelo)
				Action5 = 1                       -- Change pitchThe in 5 sec
				set(showPart, 1, 12)                     -- Stage 2 jet on
				set(Note1, "2nd Stage engine running")
			elseif get(t3) == tStart[3] + 2 then      -- pitchThe change ends
				Action5 = 0
				set(pitchThe, angThe[3])
				state = 0
			elseif get(t3) < tEnd3 then                  -- 2nd Stage running
				set(pitchThe, angThe[3])
				set(headPsi, iniPsi)
				setSpeed(3)
				if get(crrVelo) > targetV then       -- Reach target speed
					set(showPart, 0, 12)             -- Stage 2 jet off
					Action7 = 1                      -- Release 2nd Stage
					set(simStage, 2)                 -- Simulation ends
					recordParam()                    -- record current parameter
				else
					set(endPosH, math.floor(get(elv)/1000))
					if get(endPosH) > get(targetH) then   -- Reach target height
 						set(pitchThe, -89)
 						set(headPsi, iniPsi)
-- 						Action6 = 1                  -- Change pitchThe to level
						set(vy, 0.0)
						set(Note1, "Reached target height, Increasing speed")
					end
				end
			elseif get(t3) == tEnd3 then             -- 2nd Stage stops
				set(showPart, 0, 12)                 -- Stage 2 jet off
				Action7 = 1                          -- Release 2nd Stage
				setSpeed(3)
				set(simStage, 2)                     -- Simulation ends
				recordParam()                        -- record current parameter
			end
		end
		
	elseif get(simStage) == 2 then                     -- Simulation starts
	 	set(pitchThe, -89)
 		set(headPsi, iniPsi)
		set(vy, 0.0)
	end
	
	if Action1 == 1 then                           -- Tilt rockedt a little
		state = math.min( 1, (state + 0.5*get(sim_time)))
		set(pitchThe, state*angThe[1])
		if state >= 1 then
			Action1 = 0
			state = 0
		end
	end

	if Action2 == 1 then                               -- Remove Boosters
		state = math.min( 1, (get(Booster_release) + 0.5*get(sim_time)))
		set(Booster_release, state)
		if state >= 1 then
			set(hideBooster1,1)
			set(hideBooster2,1)
			Action2 = 0 
			state = 0
		end
	end
	
	if Action3 == 1 then                  -- Stage 1 Change pitchThe and headPsi
		state = math.min( 1, (state + 0.1*get(sim_time)))
		if state <= 0.5 then
			set(pitchThe, angThe[1] + 2*state*(angThe[2]-angThe[1]))
		elseif 0.5<state and state<1 then
			set(headPsi, 90 + (2*state - 1.0)*(iniPsi-90))
		else
			Action3 = 0
			set(pitchThe, angThe[2])
			set(headPsi, iniPsi)
			state = 0
		end
	end

	if Action4 == 1 then                         -- animation at Stage 1 release
		state = math.min( 1, (get(First_release) + 0.2*get(sim_time)))
		set(First_release, state)
		if state >= 1 then
			set(hideFirstS,  1)                  -- 1st Stage removed
			set(hideSecondC, 1)                  -- 2nd cover removed
			set(hideFairingL,1)
			set(hideFairingR,1)
			Action4 = 0
			state = 0
		end
	end
	
	if Action5 == 1 then                    -- Change pitchThe of 2nd Stage
		state = math.min( 1, (state + 0.5*get(sim_time)))
		set(pitchThe, angThe[2] + state*(angThe[3]-angThe[2]))
	end

	if Action6 == 1 then                       -- Change pitchThe to level
		state = math.min( 1, (state + 0.5*get(sim_time)))
		set(pitchThe, angThe[3] - state*(89+angThe[3]))
		if state >= 0.5 then
			Action6 = 0
			set(pitchThe, -89)
			state = 0
			set(vy, 0.0)
		end
	end

	if Action7 == 1 then                                     -- Remove 2nd
		state = math.min( 1, (get(Second_release) + 0.5*get(sim_time)))
		set(Second_release, state)
		if state >= 0.7 then
			set(hideSecondS, 1)
			Action7 = 0
			state = 0
		end
	end
	setOrient()	
	stopRotation()
end

-- Draw launcher  ------------------------------------------------
-- iniX = get(planeX)
-- iniY = get(planeY)
-- iniZ = get(planeZ)
-- function drawObjects()
-- 	sasl.drawObject ( LauncherObj, iniX, iniY, iniZ, 0, 90, 0, 1, 1)
-- end

-- default view setting  ------------------------------------------------
viewSet = sasl.findCommand ( "sim/view/circle" )
function onModuleInit ()
	sasl.commandOnce( viewSet )
end

function onAirportLoaded()
	sasl.commandOnce( viewSet )
end

components={ simBoard{}, simLiftoff{}, simPopup{},}